/*
* Copyright © 2018 - 2019 Virtru Corporation
*
* SPDX - License - Identifier: MIT
*
*/
//
//  Virtru TDF3 SDK
//
//  Created by Sujan Reddy on 2019/07/17.
//

#ifndef VIRTRU_TDF3_SDK_VIRTRU_CLIENT_H
#define VIRTRU_TDF3_SDK_VIRTRU_CLIENT_H

#include <string>
#include <vector>

#include <virtru_policy.h>
#include <virtru_encrypt_file_param.h>
#include <tdf3_constants.h>

/// The Client is the entry point for all the major Virtru operations, such as encrypt, decrypt
/// and policy management. The user of this class should catch virtru::Expection for any failure.

namespace virtru {

    /// Forward declaration.
    class ILogger;
    class VirtruTDF3Builder;
    class EncryptFileParam;

    class Client {
    public:
        /// Constructs a new Client instance with owner and appId.
        /// \param owner - The owner's email address to be used to authenticate for encrypt, decrypt
        /// and policy management. Any data encrypted will be owned by the supplied email address.
        /// The owner will always have access to the encrypted data (TDF). 
        /// \param appId - The virtru application ID can be obtained from the 
        /// <a href="https://developer.virtru.com/docs/how-to-download-appid-token">Virtru Dashboard</a>
        Client(std::string owner, std::string appId);

        /// Constructs a new Client instance with owner and HMAC apiKey/apiSecret.  <a href="https://developer.virtru.com/docs/how-to-add-authentication#section-3-hmac-token-and-secret">Contact Virtru</a> to get your
        /// organization's HMAC apiKey/apiSecret 
        /// \param owner - The owner's email address to be used to authenticate  for encrypt, decrypt
        /// and policy management.
        /// \param apiKey - The HMAC api key
        /// \param apiSecret - The HMAC api secret
        Client(std::string owner, std::string apiKey, std::string apiSecret);

        /// Default constructor is not supported
        Client() = delete;

        /// Destroy the Client instance.
        ~Client();

        /// Copy constructor
        Client(const Client& client) = delete;

        /// Assignment operator
        Client& operator=(const Client& client)  = delete;

        /// Move copy constructor
        Client(Client&& client) = delete;

        /// Move assignment operator
        Client& operator=(Client&& client)  = delete;

    public: /// Optional \n
        ///
        /// Note: Defaults will be used if these methods are not used. \n
        ///

        /// Set the KAS url that will be used for TDF operations.  Optional - this method should be only used
        /// if you have your own key access server infrastructure.
        /// \param kasUrl - The KAS (Key Access Server) url
        void setKasUrl(std::string kasUrl);

        /// Set the EAS url that will be used for TDF operations.  Optional - this method should be only used
        /// if you have your own entity attribute server infrastructure.
        /// \param easUrl - The EAS (Entity Attribute Server) url
        void setEasUrl(std::string easUrl);

        /// Set the ACM url that will be used for TDF policy sync operations.  Optional - this method should be only used
        /// if you have your own acm server infrastructure.
        /// \param easUrl - The ACM server url
        void setAcmUrl(std::string acmUrl);

        /// Set the secure reader url which will be used in html format TDF. This is the URL that will be used on opening
        /// the .html TDF. Optional - Should be only used if you host your own secure reader.
        /// \param url - The secure reader url
        void setSecureReaderURL(std::string srUrl);

        /// Set the TDF protocol (format) to be used for encryption and decryption operations. HTML is the
        /// default format.  The protocols refer to the data format after encryption. 
        /// Options are .tdf (which is a zip) or HTML.
        /// \param protocol - type of the protocol (format)
        void setProtocol(Protocol protocol);

        /// Set the type of key access. The types of key access are 'Remote' and 'Wrapped'.
        ///     Remote - The key is stored on remote server.
        ///     Wrapped - The key is embedded in TDF.
        /// The default is the recommended type 'Remote' 
        /// \param keyAccessType - The key access type.
        void setKeyAccessType(KeyAccessType keyAccessType);

    public: /// Optional - Logging \n
        /// Enable the internal logger class to write logs to the console for given LogLevel. The default Loglevel is
        /// 'Warn' if not specified.
        /// \param logLevel - The log level
        void enableConsoleLogging(LogLevel logLevel = LogLevel::Warn);

        /// Optional - Set the external logger instance. The internal logger will be used if the caller does not
        /// specify one.  The caller is responsible for ownership of the logger instance.\n
        /// \n NOTE: Once you use this method, the internal logger class is freed, and the internal logger will no longer be used.
        /// \param externalLogger - The external logger instance (std::weak_ptr).
        /// \param logLevel - The log level, the default is 'Warn' if not specified.
        void setExternalLogger(std::weak_ptr<ILogger> externalLogger, LogLevel logLevel = LogLevel::Warn);

    public: /// Optional - configure root CA's \n
        /// Set the cert authority which will be used in SSL handshake for all the network I/O. \n NOTE: This is 
        /// optional. If not specified, the SDK will supply a default.
        /// \param certAuthority - A string which holds the cert authority which will be used in SSL handshake
        /// for all the network I/O
        void setCertAuthority(const std::string& certAuthority);

    public: // Encrypt and Decrypt
        /// Encrypt the contents of the input file into a TDF. In the process of encryption, a policy is
        /// associated with the TDF. The policy has a unique id which can be used to identify the TDF policy.
        /// \param encryptFileParam  - Object holding all the required information for encrypt operations
        /// \return Policy uuid of the TDF, can be used later to update the policy controls
        std::string encryptFile(const EncryptFileParam& encryptFileParam);

        /// Decrypt the contents of the TDF file into its original content.
        /// \param inFilepath - The TDF file on which the decryption is performed
        /// \param outFilepath - The file path of the original content after successful decryption
        void decryptFile(std::string inFilepath, std::string outFilepath);

    public: // Update policy
        /// Update the policy for a given TDF file using its policy uuid.
        /// \param policy - The policy object with new controls
        /// \param policyUUID - The policy uuid of the TDF
        void updatePolicyForUUID(const Policy& policy, const std::string& policyUUID);

        /// Update the policy for multiple TDFs using their policy uuids.
        /// \param policy - The policy object with new controls
        /// \param policyUUIDs - The vector of policy uuids for the TDFs
        void updatePolicyForUUIDs(const Policy& policy, const std::vector<std::string>& policyUUIDs);

        /// Update the policy for the given TDF file
        /// \param policy - The policy object with new controls
        /// \param tdfFile - The TDF file path
        void updatePolicyForFile(const Policy& policy, const std::string& tdfFile);

        /// Update the policy for multiple TDF files
        /// \param policy - The policy object with new controls
        /// \param files - List of TDF files
        void updatePolicyForFiles(const Policy& policy, const std::vector<std::string>& files);

        /// Revoke access for all the users that were granted access to the TDF file associated with the specified policy. The owner
        /// of the TDF will still have access.
        /// \param policyUUID - The policy uuid of the TDF
        void revokePolicy(const std::string& policyUUID);

        /// Revoke access for all users that were granted access to the TDF files associated with the specified policies. The owner
        /// of the TDF will still have access.
        /// \param policyUUIDs - Vector of policy uuids
        void revokePolicies(const std::vector<std::string>& policyUUIDs);

        /// Revoke access for all users that were granted access to the TDF file. The owner of the TDF will still have access.
        /// \param tdfFile - The TDF file
        void revokeFile(const std::string& tdfFile);

        /// Revoke access for all users that were granted acccess to the TDF files. The owner of the TDF will still have access.
        /// \param files - Vector of TDF files.
        void revokeFiles(const std::vector<std::string>& files);

    private:
        /// A helper method to update the TDF builder;
        void updateBuilder();

    private: /// Data
        std::unique_ptr<VirtruTDF3Builder> m_virtruTDF3Builder;
    };
}



#endif // VIRTRU_TDF3_SDK_VIRTRU_CLIENT_H
