/*
* Copyright © 2018 - 2019 Virtru Corporation
*
* SPDX - License - Identifier: MIT
*
*/
//
//  Virtru TDF3 SDK
//
//  Created by Sujan Reddy on 2019/07/17.
//

#ifndef VIRTRU_TDF3_SDK_VIRTRU_POLICY_H
#define VIRTRU_TDF3_SDK_VIRTRU_POLICY_H

#include <string>
#include <vector>
#include <unordered_set>
#include <memory>

namespace virtru {

    /// Forward declaration
    class Client;
    class VirtruPolicyObject;

    class Policy {
    public:
        /// Constructor
        Policy();

        /// Destructor
        ~Policy();

        /// Copy constructor
        Policy(const Policy& policy);

        /// Assignment operator
        Policy& operator=(const Policy& policy);

        /// Move copy constructor
        Policy(Policy&& policy);

        /// Move assignment operator
        Policy& operator=(Policy&& policy);

    public: // Virtru controls

        /// Add access to the TDF file/data for the users in the list
        /// \param users - Share the TDF with the users in the vector
        void shareWithUsers(const std::vector<std::string>& users);

        /// Remove access to the TDF file/data for the users in the list
        /// \param users - Remove access to the TDF file/data for the users in the vector
        void removeUsers(const std::vector<std::string>& users);

        /// Turns on reshare policy control flag
        void enableReshare();

        /// Turns off reshare policy control flag
        void disableReshare();

        /// Turns on watermarking policy control flag
        void enableWatermarking();

        /// Turns off watermarking policy control flag
        void disableWatermarking();

        /// Turns on print policy control flag
        void enablePrint();

        /// Turns off print policy control flag
        void disablePrint();

        /// Turns on copy policy control flag
        void enableCopy();

        /// Turns off copy policy control flag
        void disableCopy();

        /// Turns on persistent protection policy control flag
        void enablePersistentProtection();

        /// Turns off persistent protection policy control flag
        void disablePersistentProtection();

        /// Turns on prevent download policy control flag
        void enablePreventDownload();

        /// Turns off prevent download policy control flag
        void disablePreventDownload();

        /// Set the expiration in minutes\n
        /// Access to the TDF data will be revoked for all users in the number of minutes specified, starting from the time this call is made.  The owner will retain access after the expiration time is reached.
        /// \param mins - minutes until expiration
        void expireInMins(unsigned mins);

        /// Set the expiration in days\n
        /// Access to the TDF data will be revoked for all users in the number of days specified, starting from the time this call is made.  The owner will retain access after the expiration time is reached.
        /// \param mins - days until expiration
        void expireInDays(unsigned days);

        /// Access to the TDF data will be revoked for all users at the specified date.  The owner will retain access after the expiration time is reached.
        /// The expiration date is specified in ISO-8601 format, and can include a time component.  Ex: 2019-05-24T16:12:41Z
        /// \param expirationDate - Expiration date when the TDF will expire
        void addExpiration(const std::string& expirationDate);

        /// Remove the expiration
        void removeExpiration();

        /// Return the description of all the policy controls of the instance.
        /// \return - A formated string of all the policy controls of the instance.
        std::string toString() const;

    private:
        /// Expresses the policy as a json string for KAS
        /// \return - Json fomated string.
        std::string toJsonStringForKas() const;

        /// Return the shared users of this policy
        /// \return - A set of users.
        std::unordered_set<std::string> getSharedWithUsers() const;

        /// Apply the policy control on to policyObject.
        /// \param policyObject - The VirtruPolicyObject.
        void applyPolicy(VirtruPolicyObject& policyObject) const;

    private: // Not supported yet
        /// Turns on one-click flag
        void enableOneClick();

        /// Turns off one-click flag
        void disableOneClick();

    private:
        friend Client;

        struct Impl;
        std::unique_ptr<Impl> m_impl;
    };
}

#endif //VIRTRU_TDF3_SDK_VIRTRU_POLICY_H
